(* runtime configuration *)
IMPLEMENTATION MODULE Config;

IMPORT Str, FIO, FIOStd, Lib2, Lib3, Args, StrOut;

CONST
  prg = 'DTC';
  ver = 3;
  ext = '2019-05-07';

  SO ::= StrOut;


PROCEDURE WrInfo( VAR b: SO.TBuffer );
BEGIN
  SO.WrStr( b, prg );
  SO.WrStr( b, ' v' );
  SO.WrCard( b, ver, 0 );
  SO.WrStr( b, ' (' );
  SO.WrStr( b, ext );
  SO.WrStr( b, '): Date/Time converter.' );
END WrInfo;


PROCEDURE ParseArgs();

  PROCEDURE Help();
  CONST
    f ::= FIOStd.out;
  VAR
    b: SO.TBuffer;
    s: ARRAY [0..127] OF CHAR;
  BEGIN
    SO.Init( b, s );
    SO.Clear( b ); SO.WrStr( b, "(* " ); WrInfo( b ); SO.WrStr( b, " *)" ); SO.WrLn( b );
    FIO.WrStr( f, s );
    SO.Clear( b ); SO.WrStr( b, "syntax = '" ); SO.WrStr( b, prg ); SO.WrStr( b, "' { option } [ date [ time ] ] ." ); SO.WrLn( b );
    FIO.WrStr( f, s );
    SO.Clear( b ); SO.WrStr( b, "option = '-h'      (* show this info and exit *)" ); SO.WrLn( b );
    FIO.WrStr( f, s );
    SO.Clear( b ); SO.WrStr( b, "       | '-d'      (* show debug messages *)" ); SO.WrLn( b );
    FIO.WrStr( f, s );
    SO.Clear( b ); SO.WrStr( b, "       | '-m'      (* machine input *)" ); SO.WrLn( b );
    FIO.WrStr( f, s );
    SO.Clear( b ); SO.WrStr( b, "       | '-v'      (* be verbose *) ." ); SO.WrLn( b );
    FIO.WrStr( f, s );
    SO.Clear( b ); SO.WrStr( b, "minDate = '1990-01-01' ." ); SO.WrLn( b );
    FIO.WrStr( f, s );
    SO.Clear( b ); SO.WrStr( b, "maxDate = '2169-06-06' ." ); SO.WrLn( b );
    FIO.WrStr( f, s );
    SO.Clear( b ); SO.WrStr( b, "minTime = '00:00:00.000' ." ); SO.WrLn( b );
    FIO.WrStr( f, s );
    SO.Clear( b ); SO.WrStr( b, "maxTime = '23:59:59.999' ." ); SO.WrLn( b );
    FIO.WrStr( f, s );
    Lib3.SetReturnCode( 0 );
    HALT;
  END Help;

  PROCEDURE Error( t-: ARRAY OF CHAR );
  VAR
    b: SO.TBuffer;
    s: ARRAY [0..127] OF CHAR;
  BEGIN
    SO.Init( b, s );
    SO.WrStr( b, prg );
    SO.WrStr( b, '.Config.ParseArgs.Error: ' );
    SO.WrStr( b, t );
    IF Args.len > 0 THEN
      SO.WrStr( b, ": '" );
      SO.WrStr( b, Args.lit );
      SO.WrStr( b, "'" );
    END;
    SO.WrLn( b );
    SO.WrStr( b, "Try '" );
    SO.WrStr( b, prg );
    SO.WrStr( b, " -h' for help." );
    Lib3.FatalError( s );
  END Error;

  PROCEDURE GetCard( VAR x: LONGCARD );
  VAR
    ok: BOOLEAN;
  BEGIN
    x := Str.StrToCard( Args.lit, 10, ok );
    IF ~ ok THEN Error( 'Invalid number' ) END;
  END GetCard;

  PROCEDURE Options();
  BEGIN
    WHILE Args.sym = Args.symOpt DO
      IF    Str.Compare( Args.lit, '?' ) = 0 THEN
        Help();
      ELSIF Str.Compare( Args.lit, 'h' ) = 0 THEN
        Help();
      ELSIF Str.Compare( Args.lit, 'd' ) = 0 THEN
        data.dbg := TRUE;
      ELSIF Str.Compare( Args.lit, 'm' ) = 0 THEN
        data.mch := TRUE;
      ELSIF Str.Compare( Args.lit, 'v' ) = 0 THEN
        data.vrb := TRUE;
      ELSE
        Error( 'Invalid option' );
      END;
      Args.Read();
    END;
  END Options;

CONST
  sep = ' ';
BEGIN
  Args.Reset();
  Args.Read();
  Options();
  WHILE Args.sym = Args.symArg DO
    Str.Append( data.arg, Args.lit );
    Str.Append( data.arg, sep );
    Args.Read();
    Options();
  END;
END ParseArgs;

PROCEDURE InitDir();
VAR
  drv: SHORTCARD;
BEGIN
  drv := FIO.GetDrive();
  FIO.GetDir( drv, data.dir );
  Str.Prepend( data.dir, ':' );
  Str.Prepend( data.dir, CHR( ORD( 'A' ) + CARDINAL(drv-1) ) );
END InitDir;

PROCEDURE Show();
CONST
  f ::= FIOStd.err;
VAR
  b: SO.TBuffer;
  s: ARRAY [0..127] OF CHAR;

  PROCEDURE Out();
  BEGIN
    FIO.WrStr( f, s ); FIO.WrLn( f );
  END Out;

  PROCEDURE ShowHdr();
  BEGIN
    SO.Clear( b ); WrInfo( b ); Out();
    SO.Clear( b ); SO.WrStr( b, prg ); SO.WrStr( b, '.Config.data:' ); Out();
  END ShowHdr;

  PROCEDURE Begin( n-: ARRAY OF CHAR );
  CONST w = 3;
  BEGIN
    SO.Clear( b ); SO.WrStr( b, '  ' ); SO.WrStrAdj( b, n, -w ); SO.WrStr( b, ' : ' );
  END Begin;

  PROCEDURE ShowStr( n-: ARRAY OF CHAR; v-: ARRAY OF CHAR );
  BEGIN
    Begin( n ); SO.WrChar( b, "'" ); SO.WrStr( b, v ); SO.WrChar( b, "'" ); Out();
  END ShowStr;

  PROCEDURE ShowCard( n-: ARRAY OF CHAR; v: CARDINAL );
  CONST w = 1;
  BEGIN
    Begin( n ); SO.WrLngCard( b, v, w ); Out();
  END ShowCard;

BEGIN
  SO.Init( b, s );
  ShowHdr();
  ShowStr(  'prg', data.prg );
  ShowCard( 'ver', data.ver );
  ShowCard( 'dbg', ORD( data.dbg ) );
  ShowCard( 'vrb', ORD( data.vrb ) );
  ShowCard( 'mch', ORD( data.mch ) );
  ShowStr(  'dir', data.dir );
  ShowStr(  'arg', data.arg );
  ShowCard( 'len', data.len );
END Show;

PROCEDURE InitModule();
BEGIN
  Lib2.Fill( data, 0 );
  Str.Copy( data.prg, prg );
  data.ver := ver;
  data.ext := ext;
  data.dbg := FALSE;
  ParseArgs();
  data.len := Str.Length( data.arg );
  InitDir();
  IF data.vrb THEN Show() END;
END InitModule;

BEGIN
  InitModule();
END Config.